cat.db = {}
cat.db.data = {}
cat.db.save_id = 64700723046



--applies a callback function to all items
function cat.db.for_all_items(func)
	for name,item in pairs(cat.db.data) do
		func(item)
	end
end



function cat.db.count_items()
	local count = 0
	cat.db.for_all_items(function(item) count = count + 1 end)
	return count
end



function cat.db.load()
	--only load if we have to, so that we can liberally sprinkle this function through the code
	if cat.db.count_items() == 0 then
		cat.db.data = unspickle(LoadSystemNotes(cat.db.save_id))
	end
end



function cat.db.save()
	SaveSystemNotes(spickle(cat.db.data), cat.db.save_id)
end



function cat.db.insert_item_from_station(item, sectorid)
	cat.db.load()
	--initialize arrays
	if cat.db.data[item.name] == nil then
		cat.db.data[item.name] = {}
	end
	if cat.db.data[item.name].sectors == nil then
		cat.db.data[item.name].sectors = {}
	end
	if sectorid ~= nil and cat.db.data[item.name].sectors[sectorid] == nil then
		cat.db.data[item.name].sectors[sectorid] = {}
	end
	
	--TODO:  nil-check all this crap
	--store the static data
	cat.db.data[item.name].name = item.name
	cat.db.data[item.name].volume = item.volume
	cat.db.data[item.name].mass = item.mass
	cat.db.data[item.name].type = item.type
	cat.db.data[item.name].sortgroup = item.sortgroup
	cat.db.data[item.name].longdesc = item.longdesc
	cat.db.data[item.name].extendeddesc = item.extendeddesc
	cat.db.data[item.name].icon = item.icon
	cat.db.data[item.name].meshfile = item.meshfile

	--update the last-seen price
	cat.db.data[item.name].lastbuyprice = item.price

	--store the sector-specific data
	cat.db.data[item.name].sectors[sectorid].neededlevels = item.neededlevels
	cat.db.data[item.name].sectors[sectorid].lastbuyprice = item.price
end



function cat.db.insert_items_from_station()
	cat.db.load()
	for itemid = 1, GetNumStationMerch() do
		cat.db.insert_item_from_station(GetStationMerchInfo(itemid), GetCurrentSectorid())
	end
	cat.db.save()
end



function cat.db.insert_item_from_inventory(cargoid)
	cat.db.load()
	
	--get item name, which we use as the index in the db
	local name = GetInventoryItemName(cargoid)
	if name == nil then
		return
	end

	--initialize array
	if cat.db.data[name] == nil then
		cat.db.data[name] = {}
	end

	--store the static data
	cat.db.data[name].name = name
	cat.db.data[name].volume = GetInventoryItemVolume(cargoid)
	cat.db.data[name].mass = GetInventoryItemMass(cargoid)
	cat.db.data[name].type = GetInventoryItemType(cargoid)
	--cat.db.data[name].sortgroup = 
	--cat.db.data[name].longdesc = 
	cat.db.data[name].extendeddesc = GetInventoryItemExtendedDesc(cargoid)
	cat.db.data[name].icon = GetInventoryItemIcon(cargoid)
	--cat.db.data[name].meshfile = 

	--update the last-seen price
	local totalprice, locationid = GetBestPriceInfoOfItem(cargoid)
	if totalprice ~= nil then
		local bestprice = totalprice/GetInventoryItemQuantity(cargoid)
		cat.db.data[name].lastsellprice = bestprice

		if locationid ~= nil then
			local sectorid = (locationid-1)/100
			--initialize array
			if cat.db.data[name].sectors == nil then
				cat.db.data[name].sectors = {}
			end
			if cat.db.data[name].sectors[sectorid] == nil then
				cat.db.data[name].sectors[sectorid] = {}
			end

			--store the sector-specific data
			cat.db.data[name].sectors[sectorid].lastsellprice = bestprice
			--TODO:  Store price history
		end
	end
end



function cat.db.insert_items_from_inventory(data)
	cat.db.load()
	cat.db.insert_item_from_inventory(data)
	cat.db.save()
end



--use this outside db.lua get item data, rather than directly querying the data structure (since that might change)
function cat.db.get_item_by_name(name)
	cat.db.load()
	
	if cat.db.data[name] ~= nil then
		local item = {}
		item.name = cat.db.data[name].name
		item.volume = cat.db.data[name].volume
		item.mass = cat.db.data[name].mass
		item.lastbuyprice = cat.db.data[name].lastbuyprice
		item.lastsellprice = cat.db.data[name].lastsellprice
		item.type = cat.db.data[name].type
		item.sortgroup = cat.db.data[name].sortgroup
		item.longdesc = cat.db.data[name].longdesc
		item.extendeddesc = cat.db.data[name].extendeddesc
		item.icon = cat.db.data[name].icon
		item.meshfile = cat.db.data[name].meshfile
		--item.neededlevels = cat.db.data[name].sectors[sectorid].neededlevels

		return item
	else
		return nil
	end
end



function cat.db.only_in_system(sectors, systemname)
	local ret = {}
	for sectorid,v in pairs(sectors) do
		local coords = ShortLocationStr(sectorid)
		if (string.find(string.lower(coords), string.lower(systemname)) ~= nil) then
			ret[sectorid] = v
		end
	end
	return ret
end



--use this outside db.lua get an item's sector data, rather than directly querying the data structure (since that might change)
--if 'systemname' is set, limit to stations within that system
function cat.db.get_item_sectors_by_name(name, systemname)
	cat.db.load()
	
	if cat.db.data[name] ~= nil then
		local sectors = {}
		sectors = cat.db.data[name].sectors
		
		if systemname ~= nil then
			sectors = cat.db.only_in_system(sectors, systemname)
		end

		return sectors
	else
		return nil
	end
end



function cat.db.get_items_by_pattern(pattern)
	cat.db.load()
	
	local items = {}
	cat.db.for_all_items( function(item)
		if (string.find(string.lower(item.name), string.lower(pattern)) ~= nil) then
			items[#items+1] = item
		end
	end
	)
	return items
end



function cat.db.get_average_buy_price(name)
	--if item does not have sufficient data, return nil
	if cat.db.data[name] == nil or cat.db.data[name].lastbuyprice == nil then
		return nil
	end
	
	--compute average
	local totalprice = 0
	local totalsectors = 0
	for sectorid,sector in pairs(cat.db.get_item_sectors_by_name(name)) do
		if sector.lastbuyprice ~= nil then
			totalprice = totalprice + sector.lastbuyprice
			totalsectors = totalsectors + 1
		end
	end
	return (totalprice/totalsectors)
end



function cat.db.get_average_sell_price(name)
	--if item does not have sufficient data, return nil
	if cat.db.data[name] == nil or cat.db.data[name].lastsellprice == nil then
		return nil
	end
	
	--compute average
	local totalprice = 0
	local totalsectors = 0
	for sectorid, sector in pairs(cat.db.get_item_sectors_by_name(name)) do
		if sector.lastsellprice ~= nil then
			totalprice = totalprice + sector.lastsellprice
			totalsectors = totalsectors + 1
		end
	end
	return (totalprice/totalsectors)
end



function cat.db.get_stats(name)
	local item = cat.db.get_item_by_name(name)
	if name ~= nil and item == nil then
		--item is not in the database, give up
		return nil
	end
	local desc
	if (item.extendeddesc) then
		desc = item.extendeddesc
	elseif (item.longdesc) then
		desc = item.longdesc
	else
		--item doesn't have any stats in the db
		return nil
	end
	
	local stats = {}

	stats['damage'] = desc:match("Damage: +([^\r\n|]+)")
	stats['repair'] = desc:match("Repairs: +([^\r\n|]+)")
	stats['splash'] = desc:match("Splash Radius: +([^\r\n|]+)")
	stats['capacity'] = desc:match("^Capacity: +([^\r\n|]+)")
	stats['detonation'] = desc:match("Detonation: +([^\r\n|]+)")
	if not stats['detonation'] then
		stats['detonation'] = desc:match("proximity +([^\r\n|]+)")
	end
	stats['fuel'] = desc:match("Fuel: +([^\r\n|]+)")
	stats['volley'] = desc:match("Volley: +([^\r\n|]+)")
	stats['maneuverability'] = desc:match("Maneuverability: +([^\r\n|]+)")
	stats['velocity'] = desc:match("Velocity: +([^\r\n|]+)")
	stats['energy'] = desc:match("Energy: +([^\r\n|]+)")
	stats['generation'] = desc:match("Generation: +([^\r\n|]+)")
	stats['delay'] = desc:match("Delay: +([^\r\n|]+)")
	stats['grid'] = desc:match("Grid Usage: +([^\r\n|]+)")
	stats['grid power'] = desc:match("Grid Power: +([^\r\n|]+)")
	stats['mass'] = desc:match("Mass: +([^\r\n|]+)")
	stats['range'] = desc:match("Range: +([^\r\n|]+)")
	stats['rate'] = desc:match("^Rate: +([^\r\n|]+)")
	stats['charge-rate'] = desc:match("Charge Rate: +([^\r\n|]+)")
	stats['lifetime'] = desc:match("Lifetime: +([^\r\n|]+)")
	stats['heat'] = desc:match("Heat: +([^\r\n|]+)")
	stats['armor'] = desc:match("Armor: +([0-9]+)")
	stats['cargo'] = desc:match("Cargo Capacity: +([^\r\n|]+)")
	stats['s-ports'] = desc:match("Weapons: +.*([0-9]+) +Small")
	stats['l-ports'] = desc:match("Weapons: +.*([0-9]+) +Large")
	stats['length'] = desc:match("Length: +([^\r\n|]+)")
	stats['thrust'] = desc:match("Thrust: +([^\r\n|]+)")
	stats['speed'] = desc:match("Max Speed: +([^\r\n|]+)")
	stats['spin'] = desc:match("Spin torque: +([^\r\n|]+)")
	stats['turbo-thrust'] = desc:match("Turbo Thrust: +([^\r\n|]+)")
	stats['turbo-speed'] = desc:match("Turbo max speed: +([^\r\n|]+)")
	stats['turbo-energy'] = desc:match("Turbo energy: +([^\r\n|]+)")
	
	--these are for specialized mining beams
	stats['aquean'] = desc:match("Aquean: +([^\r\n|]+)")
	stats['silicate'] = desc:match("Silicate: +([^\r\n|]+)")
	stats['carbonic'] = desc:match("Carbonic: +([^\r\n|]+)")
	stats['ferric'] = desc:match("Ferric: +([^\r\n|]+)")
	stats['ishik'] = desc:match("Ishik: +([^\r\n|]+)")
	stats['vanazek'] = desc:match("VanAzek: +([^\r\n|]+)")
	stats['xithricite'] = desc:match("Xithricite: +([^\r\n|]+)")
	stats['lanthanic'] = desc:match("Lanthanic: +([^\r\n|]+)")
	stats['denic'] = desc:match("Denic: +([^\r\n|]+)")
	stats['pyronic'] = desc:match("Pyronic: +([^\r\n|]+)")
	stats['apicene'] = desc:match("Apicene: +([^\r\n|]+)")
	stats['pentric'] = desc:match("Pentric: +([^\r\n|]+)")
	stats['heliocene'] = desc:match("Heliocene: +([^\r\n|]+)")

	if (stats.damage and stats.energy) then
		stats.dpe = cat.db.round(cat.db.getnum(stats.damage)/cat.db.getnum(stats.energy),2)
	end
	if (stats.damage and stats.delay) then
		stats.dps = cat.db.round(cat.db.getnum(stats.damage)/cat.db.getnum(stats.delay),2)
	end
	if (stats.damage and stats.volley) then
		stats.dpv = cat.db.getnum(stats.damage)*cat.db.getnum(stats.volley)
	end
	if (stats.damage and stats.capacity) then
		stats.totaldamage = cat.db.getnum(stats.damage)*cat.db.getnum(stats.capacity)
	end

	return stats
end

function cat.db.getnum(s)
	s = tostring(s)
	return(tonumber(s:match("([.0-9]+)")))
end

function cat.db.round(val, decimal)
	if (val) then
		if (decimal) then
			return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
		else
			return math.floor(val+0.5)
		end
	end
end

function cat.db.set_special(name, reason)
	cat.db.load()
	--initialize arrays
	if cat.db.data[name] == nil then
		cat.db.data[name] = {}
	end
	cat.db.data[name].name = name
	cat.db.data[name].special = true
	if reason ~= nil then
		cat.db.data[name].special_reason = reason
	else
		cat.db.data[name].special_reason = "special"
	end
	cat.db.save()
end

function cat.db.unset_special(name)
	cat.db.load()
	--initialize arrays
	if cat.db.data[name] == nil then
		cat.db.data[name] = {}
	end
	cat.db.data[name].name = name
	cat.db.data[name].special = false
	cat.db.data[name].special_reason = nil
	cat.db.save()
end

function cat.db.is_special(name)
	cat.db.load()
	if cat.db.data[name] == nil or cat.db.data[name].special == nil then
		return false
	end
	return cat.db.data[name].special
end

function cat.db.get_special_reason(name)
	cat.db.load()
	if cat.db.data[name] == nil or cat.db.data[name].special == nil or cat.db.data[name].special_reason == nil then
		return ""
	end
	return cat.db.data[name].special_reason
end

function cat.db.get_specials()
	cat.db.load()

	local items = {}
	cat.db.for_all_items( function(item)
		if item.special ~= nil and item.special then
			items[#items+1] = item
		end
	end
	)
	return items
end

