--Catalog, by Pizzasgood (Rin Ganborro)
--This is a plugin to help you know what loot is worth, and also for
--tracking which stations sell what, and what the stats for items are.
--Run "/cat" for a list of commands, or read the README

cat = {}
cat.active = true
cat.version = 1.4

--load the libraries
dofile("db.lua")
dofile("out.lua")



function cat.toggle(state)
	if state ~= nil then
		cat.active = state
	else
		cat.active = not cat.active
	end
	
	if cat.active then
		cat.out.print("\127ffffffcatalog: active")
	else
		cat.out.print("\127ffffffcatalog: disabled")
	end
end



--update the database each time we dock
function cat.OnEnteredStation(event, data)
	if cat.active then
		cat.db.insert_items_from_station()
	end
end
RegisterEvent(cat.OnEnteredStation, "ENTERED_STATION");



--update the database and display info each time we pick up cargo
cat.cargotimer = Timer();
function cat.OnInventory(event, data)
	if cat.active then
		--need a delay, because apparently we can't check the best-price data immediately
		cat.cargotimer:SetTimeout(500,function() cat.db.insert_items_from_inventory(data); cat.out.print_inventory(data) end);
	end
end
RegisterEvent(cat.OnInventory, "INVENTORY_ADD");
RegisterEvent(cat.OnInventory, "INVENTORY_RECEIVED");



--display information each time a ship is scanned
--TODO:  check if this works on roids too - probably needs adjustments to name-detection, and probably needs radar.GetRadarSelectionID() instead of GetTargetInfo()
function cat.OnTargetScanned(event, data)
	if cat.active then
		if data ~= nil then
			--make sure we didn't just scan this dude
			local target = GetTargetInfo()
			if (cat.last_scanned == nil or cat.last_scanned ~= target) then
				cat.last_scanned = target
				--loop through his cargo and display printouts
				for raw_name in string.gmatch(data, "[^\n]*") do
					local name = string.gsub(raw_name, " x%d+", "")
					local number = string.gsub(raw_name, ".*x(%d+).*", "%1")
					number = tonumber(number)
					if number == nil then
						number = 1
					end
					cat.out.print_stack(name, number)
				end
			end
		end
	end
end
RegisterEvent(cat.OnTargetScanned, "TARGET_SCANNED");



--display information each time an item is targeted
function cat.OnTargetChanged(event, data)
	if cat.active then
		local target = GetTargetInfo()
		if target ~= nil then
			local name = string.gsub(target, " *%(.+cu%)", "")
			local number = string.gsub(target, ".*%((%d+)x.*cu%).*", "%1")
			number = tonumber(number)
			if number == nil then
				number = 1
			end
			--TODO:  Use a timer and a list of recent ores, or maybe list of ships scanned or something.
			if name ~= "" and name ~= "\n" then
				cat.out.print_stack(name, number)
			end
		end
	end
end
RegisterEvent(cat.OnTargetChanged, "TARGET_CHANGED");


--cli
function cat.cmd(_,data)
	if data == nil then
		cat.out.usage()
	elseif data[1] == "on" then
		cat.toggle(true)
	elseif data[1] == "off" then
		cat.toggle(false)
	elseif data[1] == "toggle" then
		cat.toggle()
	elseif data[1] == "p" and data[2] then
		cat.out.print_default(data[2])
	elseif data[1] == "pp" and data[2] then
		for i,item in pairs( cat.db.get_items_by_pattern(data[2]) ) do
			cat.out.print_default(item.name)
		end
	elseif data[1] == "w" and data[2] then
		cat.out.print_locations(data[2], data[3])
	elseif data[1] == "wp" and data[2] then
		for i,item in pairs( cat.db.get_items_by_pattern(data[2]) ) do
			cat.out.print_locations(item.name, data[3])
		end
	elseif data[1] == "l" and data[2] then
		cat.out.print_info(data[2])
	elseif data[1] == "lp" and data[2] then
		for i,item in pairs( cat.db.get_items_by_pattern(data[2]) ) do
			cat.out.print_info(item.name)
		end
	elseif data[1] == "ll" and data[2] then
		cat.out.print_long_info(data[2])
	elseif data[1] == "llp" and data[2] then
		for i,item in pairs( cat.db.get_items_by_pattern(data[2]) ) do
			cat.out.print_long_info(item.name)
		end
	elseif data[1] == "c" and data[2] and data[3] then
		cat.out.compare(data[2], data[3])
	elseif data[1] == "cp" and data[2] and data[3] then
		cat.out.compare_pattern(data[2], data[3])
	elseif data[1] == "s" and data[2] then
		if data[3] then
			cat.db.set_special(data[2], data[3])
		else
			cat.db.set_special(data[2])
		end
	elseif data[1] == "us" and data[2] then
		cat.db.unset_special(data[2])
	elseif data[1] == "ls" then
		cat.out.list_special()
	elseif data[1] == "debug_clear" then
		cat.db.data = {}
		cat.db.save()
		cat.out.print("cat-db cleared")
	elseif data[1] == "debug_pall" then
		cat.db.for_all_items(function(item) print(item.name) end)
	else
		cat.out.usage()
	end
end
RegisterUserCommand("cat",cat.cmd)
