cat.out = {}

--define the colors
cat.out.colors = {}
cat.out.colors.item	= "\127ffffbb"
cat.out.colors.number	= "\127ffffff"
cat.out.colors.misc	= "\127bbbbbb"
cat.out.colors.special	= "\127ff00ff"
cat.out.colors.prices = {
		"\127ff0000",
		"\127ff8800",
		"\127ffff00",
		"\12788ff00",
		"\12700ff00",
		"\12700ff88",
		"\12700ffff",
		"\1270088ff",
}

--function to color prices based on how good they are
function cat.out.colors.priceequation(price)
	local factor = 500

	--sanity check
	if price == nil then
		return nil
	end
	
	local function log2(number)
		return(math.log(number)/math.log(2))
	end
	
	--compute index
	local index = 1 + log2(math.max(price,1)/factor)
	--constrain it to the range [1,#cat.out.colors.prices]
	return(math.min(#cat.out.colors.prices, math.max(1, math.floor(index))))
end	



function cat.out.colors.price(price)
	local index = cat.out.colors.priceequation(price)
	if index ~= nil then
		return(cat.out.colors.prices[index] .. math.floor(price) .. cat.out.colors.misc)
	else
		return nil
	end
end



function cat.out.colors.bulkprice(price)
	local bulk_factor = 120
	local index = cat.out.colors.priceequation(price/bulk_factor)
	if index ~= nil then
		return(cat.out.colors.prices[index] .. math.floor(price) .. cat.out.colors.misc)
	else
		return nil
	end
end



function cat.out.colors.station(sectorid)
	local name = ShortLocationStr(sectorid)
	local stationid = sectorid * 256 + 1
	local factionid = GetStationFaction(stationid)
	local color = rgbtohex(ShipPalette_string[FactionColor[factionid]]) --cat.out.colors.faction[factionid]
	return(color .. name .. cat.out.colors.misc)
end



--define standard messages
cat.out.msg = {}
cat.out.msg.default = "%name% :\t%lastsellpricepercu% / %averagesellpricepercu% c/cu,\tbuy for %averagebuypricepercu% c/cu  %special%"
cat.out.msg.stack = "%name% :\t%lastsellpricepercu% / %averagesellpricepercu% c/cu,\tbuy for %averagebuypricepercu% c/cu,\ttotal for %number% units: %totalestprice% c  %special%"
cat.out.msg.buy_location = "%name% :\t%lastbuypriceat%c at %sector%"



--parse a string to replace %-enclosed variables with their value, and colorize prices and names
--data is a structure that can contain 'name', 'number', 'sectorid', 'fail_on_nil'
function cat.out.process(raw_message, data)
	local name;
	if data == nil or data.name == nil then
		name = data
	else
		name = data.name
	end
	
	--check the validity of the item first, and grab it now to make subsequent code cleaner
	local item = cat.db.get_item_by_name(name)
	if name ~= nil and item == nil then
		--item is not in the database, give up
		return nil
	end

	--define patterns and how to process them
	local patterns = {
		{ '%%name%%',			function() return( cat.out.colors.item .. name .. cat.out.colors.misc ) end },
		{ '%%number%%',			function() return( cat.out.colors.number .. data.number .. cat.out.colors.misc ) end },
		{ '%%sector%%',			function() return( cat.out.colors.station(data.sectorid) ) end },
		{ '%%lastbuyprice%%',		function() return( cat.out.colors.price( item.lastbuyprice ) ) end },
		{ '%%lastsellprice%%',		function() return( cat.out.colors.price( item.lastsellprice ) ) end },
		{ '%%lastbuypricepercu%%',	function() local price = item.lastbuyprice; if price ~= nil then return( cat.out.colors.price( price / item.volume ) ) else return(nil) end end },
		{ '%%lastsellpricepercu%%',	function() local price = item.lastsellprice; if price ~= nil then return( cat.out.colors.price( price / item.volume ) ) else return(nil) end end },
		{ '%%averagebuyprice%%',		function() return( cat.out.colors.price( cat.db.get_average_buy_price(name) ) ) end },
		{ '%%averagesellprice%%',		function() return( cat.out.colors.price( cat.db.get_average_sell_price(name) ) ) end },
		{ '%%averagebuypricepercu%%',	function() local price = cat.db.get_average_buy_price(name); if price ~= nil then return( cat.out.colors.price( price / item.volume ) ) else return(nil) end end },
		{ '%%averagesellpricepercu%%',	function() local price = cat.db.get_average_sell_price(name); if price ~= nil then return( cat.out.colors.price( price / item.volume ) ) else return(nil) end end },
		{ '%%lastbuypriceat%%',		function() return( cat.out.colors.price( cat.db.get_item_sectors_by_name(name)[data.sectorid].lastbuyprice ) ) end },
		{ '%%lastsellpriceat%%',	function() return( cat.out.colors.price( cat.db.get_item_sectors_by_name(name)[data.sectorid].lastsellprice ) ) end },
		{ '%%totalestprice%%',		function() local price; if item.lastsellprice ~= nil then price = item.lastsellprice elseif item.lastbuyprice ~= nil then price = cat.db.get_average_buy_price(name) else return nil end; return( cat.out.colors.bulkprice( price * data.number ) ) end },
		{ '%%special%%',		function() local special = cat.db.is_special(name); if special then return( cat.out.colors.special .. '*' .. cat.db.get_special_reason(name) .. '*' .. cat.out.colors.misc ) else return('') end end },
	}
	
	--do the actual processing
	local message = raw_message
	for i,entry in pairs(patterns) do
		if (string.find(message, entry[1]) ~= nil) then
			local replacement = entry[2]()
			if replacement ~= nil then
				message = string.gsub(message, entry[1], replacement)
			else
				--something failed (insufficient data?)
				if data.fail_on_nil then
					return nil
				else
					message = string.gsub(message, entry[1], "???")
				end
			end
		end
	end
	return(cat.out.colors.misc .. message)
end



--process and then print a string
function cat.out.print(raw_message, data)
	local message = cat.out.process(raw_message, data)
	if message ~= nil then
		print(message)
	end
end



function cat.out.print_default(name)
	cat.out.print(cat.out.msg.default, name)
end



function cat.out.print_stack(name, number)
	cat.out.print(cat.out.msg.stack, {name=name; number=number})
end



--TODO:  have this one (optionally) more like my percuprof plugin
function cat.out.print_inventory(data)
	local cargo_array={}
	if data ~= nil then
		cargo_array[1] = data
	else
		cargo_array = GetShipInventory(GetActiveShipID()).cargo
	end

	for i,cargoid in pairs(cargo_array) do
		local name = GetInventoryItemName(cargoid)
		local number = GetInventoryItemQuantity(cargoid)
		cat.out.print_stack(name, number)
	end
end


function cat.out.print_locations(name, systemname)
	local sectors = cat.db.get_item_sectors_by_name(name, systemname)
	if sectors then
		for sectorid,sector in pairs(sectors) do
			if sector.lastbuyprice ~= nil then
				cat.out.print(cat.out.msg.buy_location, {name=name; sectorid=sectorid})
			end
		end
	end
end


function cat.out.print_info(name)
	local item = cat.db.get_item_by_name(name)
	if name ~= nil and item == nil then
		--item is not in the database, give up
		print(cat.out.colors.misc .. "No such item in the database.")
		return nil
	end
	local stats = cat.db.get_stats(name)
	if stats == nil then
		--item is in the database, but doesn't have recorded stats, give up
		print(cat.out.colors.misc .. "Requested item has no stats recorded.")
		return nil
	end
	
	
	print(cat.out.colors.item .. item.name)
	print(cat.out.colors.misc .. "volume: " .. cat.out.colors.number .. item.volume)
	if (item.lastbuyprice) then
		print(cat.out.colors.misc .. "last buy price: " .. cat.out.colors.price(item.lastbuyprice))
	end
	if (item.lastsellprice) then
		print(cat.out.colors.misc .. "last sell price: " .. cat.out.colors.price(item.lastsellprice))
	end

	
	local output = cat.out.colors.misc
	local n = 0
	local limit = 8
	for field, value in pairs(stats) do
		if (n > 0) then
			output = output .. " | "
		end
		output = output .. field .. ": " .. cat.out.colors.number .. value ..  cat.out.colors.misc
		n = n + 1

		if (n >= limit) then
			print(output)
			output = cat.out.colors.misc
			n=0
		end
	end
	if (n > 0) then
		print(output)
	end
end


function cat.out.print_long_info(name)
	local item = cat.db.get_item_by_name(name)
	if name ~= nil and item == nil then
		--item is not in the database, give up
		print(cat.out.colors.misc .. "No such item in the database.")
		return nil
	end
	local stats = cat.db.get_stats(name)
	if stats == nil then
		--item is in the database, but doesn't have recorded stats, give up
		print(cat.out.colors.misc .. "Requested item has no stats recorded.")
		return nil
	end
	
	print(cat.out.colors.misc .. "NAME: " .. cat.out.colors.item .. item.name)
	print(cat.out.colors.misc .. "VOLUME: " .. cat.out.colors.number .. item.volume)
	if (item.lastbuyprice) then
		print(cat.out.colors.misc .. "LAST BUY PRICE: " .. cat.out.colors.price(item.lastbuyprice))
	end
	if (item.lastsellprice) then
		print(cat.out.colors.misc .. "LAST SELL PRICE: " .. cat.out.colors.price(item.lastsellprice))
	end
	if (item.type) then
		print(cat.out.colors.misc .. "TYPE: " .. cat.out.colors.number .. item.type)
	end
	if (item.sortgroup) then
		print(cat.out.colors.misc .. "SORTGROUP: " .. cat.out.colors.number .. item.sortgroup)
	end
	if (item.longdesc) then
		print(cat.out.colors.misc .. "LONGDESC: " .. cat.out.colors.number .. item.longdesc)
	end
	if item.extendeddesc and (not item.longdesc or item.longdesc ~= item.extendeddesc) then
		print(cat.out.colors.misc .. "EXTENDEDDESC: " .. cat.out.colors.number .. item.extendeddesc)
	end
	--print(cat.out.colors.misc .. "ICON: " .. item.icon)
	--print(cat.out.colors.misc .. "MESHFILE: " .. item.meshfile)
	print(cat.out.colors.misc .. " ---- ")
	for field, value in pairs(stats) do
		print(cat.out.colors.misc .. field .. ": " .. cat.out.colors.number .. value)
	end
end


--just like cat.out.compare() below, except it takes patterns instead of names
--it requires that each pattern only matches one result though
function cat.out.compare_pattern(pat1, pat2)
	local matches1 = cat.db.get_items_by_pattern(pat1);
	local matches2 = cat.db.get_items_by_pattern(pat2);
	local fail = false;
	if #matches1 ~= 1 then
		fail = true;
		if #matches1 > 1 then
			print(cat.out.colors.misc .. "Pattern #1 matched multiple results.  Please be more specific.  These were the matches:");
			for i,item in pairs( matches1 ) do
				print(cat.out.colors.misc .. item.name);
			end
		else
			print(cat.out.colors.misc .. "Pattern #1 failed to match anything.");
		end
	end
	if #matches2 ~= 1 then
		fail = true;
		if #matches2 > 1 then
			print(cat.out.colors.misc .. "Pattern #2 matched multiple results.  Please be more specific.  These were the matches:");
			for i,item in pairs( matches2 ) do
				print(cat.out.colors.misc .. item.name);
			end
		else
			print(cat.out.colors.misc .. "Pattern #2 failed to match anything.");
		end
	end
	if fail then
		return
	end

	cat.out.compare(matches1[1].name, matches2[1].name);
end


--compare item1 to item2 - output is basically item1-item2
function cat.out.compare(name1, name2)
	local item1 = cat.db.get_item_by_name(name1)
	if name1 ~= nil and item1 == nil then
		--item is not in the database, give up
		return nil
	end
	local stats1 = cat.db.get_stats(name1)
	if stats1 == nil then
		--item is in the database, but doesn't have recorded stats, give up
		return nil
	end
	
	local item2 = cat.db.get_item_by_name(name2)
	if name2 ~= nil and item2 == nil then
		--item is not in the database, give up
		return nil
	end
	local stats2 = cat.db.get_stats(name2)
	if stats2 == nil then
		--item is in the database, but doesn't have recorded stats, give up
		return nil
	end
	
	print(cat.out.colors.item .. name1 .. cat.out.colors.misc .. " vs. " .. cat.out.colors.item .. name2)
	

	local output = cat.out.colors.misc
	local n = 0
	local limit = 8
	for field, value1 in pairs(stats1) do
		local value2 = stats2[field]
		local polarity = true
		if (field == "energy" or field == "delay" or field == "grid" or field == "mass" or field == "heat" or field == "length" or field == "turbo-energy") then
			polarity = false
		end
		local diff = cat.out.diff(cat.db.getnum(value1),cat.db.getnum(value2), polarity)
		
		if (diff) then
			if (n > 0) then
				output = output .. " | "
			end
			output = output .. field .. ": " .. diff .. cat.out.colors.misc
			n = n + 1
		end

		if (n >= limit) then
			print(output)
			output = cat.out.colors.misc
			n=0
		end
	end
	
	for field, value2 in pairs(stats2) do
		if (not stats1[field]) then
			if (n > 0) then
				output = output .. " | "
			end
			
			local polarity = true
			if (field == "energy" or field == "delay" or field == "grid" or field == "mass" or field == "heat" or field == "length" or field == "turbo-energy") then
				polarity = false
			end
			local diff = cat.out.diff(0, cat.db.getnum(value2), polarity)
			
			output = output .. field .. ": " .. diff .. cat.out.colors.misc
			n = n + 1

			if (n >= limit) then
				print(output)
				output = cat.out.colors.misc
				n=0
			end
		end
	end
	if (n > 0) then
		print(output)
	end

	
end

--if polarity == true, + is good, - is bad
function cat.out.diff(value1, value2, polarity)
	if (value1 == nil) then
		value1 = 0
	end
	if (value2 == nil) then
		value2 = 0
	end
	local good_color = "\12700FF00"
	local bad_color = "\127FF0000"
	local positive_color
	local negative_color
	if (polarity) then
		positive_color = good_color
		negative_color = bad_color
	else
		positive_color = bad_color
		negative_color = good_color
	end
	local diff
	if (value2) then
		if (value2 > value1) then
			diff = negative_color .. "-" .. cat.db.round(value2 - value1, 2)
		elseif (cat.db.getnum(value2) < cat.db.getnum(value1)) then
			diff = positive_color .. "+" .. cat.db.round(value1 - value2, 2)
		end
	else
		diff = positive_color .. "+" .. cat.db.round(value1)
	end
	return(diff)
end


function cat.out.list_special()
	local items = cat.db.get_specials()
	for i,item in pairs(items) do
		cat.out.print_default(item.name)
	end
end


function cat.out.usage()
		cat.out.print("Usage:\
/cat on|off|toggle     : enable/disable catalog plugin\
/cat p <name>          : print average prices of item <name>\
/cat pp <pattern>      : print average prices of items with names matching <pattern>\
/cat w <name> [system] : print stations selling item <name>, optionally limited to the system [system]\
/cat wp <name> [system] : print stations selling items with names matching <pattern>, optionally limited to the system [system]\
/cat l <name>          : print short info about item <name>\
/cat lp <name>         : print short info about items with names matching <pattern>\
/cat ll <name>         : print long info about item <name>\
/cat llp <name>        : print long info about items with names matching <pattern>\
/cat c <name1> <name2> : print comparison between <name1> and <name2>\
/cat cp <pat1> <pat2>  : print comparison between patterns <pat1> and <pat2>\
/cat s <name> <reason> : flag item <name> as special, for <reason>\
/cat us <name>         : remove the special flag from item <name>\
/cat ls                : list items flagged as special\
/cat debug_clear       : clear db\
/cat debug_pall        : print names of all items in db")
end


